<?php

require_once('Archive.php');
if (file_exists(dirname(__FILE__) . '/../../../tools/pclzip/pclzip.lib.php')) {
    require_once(dirname(__FILE__) . '/../../../tools/pclzip/pclzip.lib.php');
} else {
    require_once(dirname(__FILE__).'/pclzip/pclzip.lib.php');
}

function generateArchive($themeName, $userThemeName = '', $includeThemler = true)
{
    register_shutdown_function('fatalErrorShutdownHandler');
    $archiveFile = getThemeArchive($themeName, $userThemeName, $includeThemler);
    $archiveContent = array();

    if (is_readable($archiveFile)) {
        $archiveContent = array(
            'ext' => 'zip',
            'content' => base64_encode(FilesHelper::readFile($archiveFile))
        );
        FilesHelper::deleteFile($archiveFile);
    }
    return $archiveContent;
}

function getThemeArchive($themeName, $userThemeName, $includeThemler) {
    $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
    if (!file_exists($themeDir))
        die('Error : No Theme Folder');

    $userThemeName = $userThemeName ? $userThemeName: $themeName;
    $user_template_dir = FilesHelper::normalizePath(_PS_DOWNLOAD_DIR_ . '/' . $userThemeName);
    FilesHelper::copyRecursive($themeDir, $user_template_dir);

    if (!$includeThemler) {
        $dirs = array('export', 'project');
        clearDirs($user_template_dir, $dirs);
    }

    $previewThemeDir = FilesHelper::normalizePath(getThemeDir($themeName . _PREVIEW_SUFFIX_));
    $base_upload_dir = FilesHelper::normalizePath(_PS_DOWNLOAD_DIR_);
    $archive_name = 'theme_' . uniqid(time()) .  '.zip';
    $archive_file = $base_upload_dir . '/' . $archive_name;
    $archive = new PclZip($archive_file);
    add_version_folders($archive, $base_upload_dir, $user_template_dir, $previewThemeDir, $userThemeName, $includeThemler);
    add_files_to_archive_root($archive, $base_upload_dir, $user_template_dir, $previewThemeDir, $userThemeName);

    // Temporary fix to support Prestashop Cloud Install
    deleteFromArchive($archive, 'modules/designer/override/controllers/admin/AdminLoginController.php');

    FilesHelper::removeDir($user_template_dir, true);

    return $archive_file;
}

function add_version_folders(&$archive, $base_upload_dir, $user_template_dir, $preview_template_folder, $userThemeName, $includeThemler)
{
    $preview_files = array();
    foreach (get_support_versions() as $version) {
        $preview_files[$version] = FilesHelper::enumerateFiles($preview_template_folder . '/' . $version);
    }
    $preview_name = $userThemeName . _PREVIEW_SUFFIX_;
    $version_folders = get_support_versions_folder($user_template_dir);
    $cms_version = get_cms_version();

    $dirs = array('themes', 'modules');
    $modules = array('designerpreview', 'blocktopmenu_mod');
    if ($includeThemler) $modules[] = 'designer'; 

    foreach ($version_folders  as $version => $folder) {
        $root = $base_upload_dir . (strpos($cms_version, $version) !== false ? '' : '/' . $version);

        foreach ($dirs as $name) {
            $dir = $root . '/' . $name;
            $arch_dirs[$name] = $dir;
            FilesHelper::createDir($dir);
        }

        FilesHelper::copyRecursive($user_template_dir, $arch_dirs['themes'] . '/' . $userThemeName);
        foreach ($modules as $module)
            FilesHelper::copyRecursive(_PS_MODULE_DIR_ . $module, $arch_dirs['modules'] . '/' . $module);

        // current version files in preview folder doesn't need to be copied
        if (strpos($cms_version, $version) !== false)
            continue;

        $files = $preview_files[$version];
        $pattern = '/(.*?\/themes\/'.$preview_name.'\/'.$version.'\/)(.*?)/si';

        foreach ($files as $file) {
            $path = $file['path'];
            $isThemler = strpos($path, 'designer/');
            if ($isThemler && !$includeThemler) 
                continue;

            $isModule = $isThemler || strpos($path, 'designerpreview/') || strpos($path, 'blocktopmenu_mod/');
            $to_replace = ($isModule ? $arch_dirs['modules'] : $arch_dirs['themes'].'/'.$userThemeName) . '/$2';
        
            $r_path = preg_replace($pattern, $to_replace, $path);
            $r_path = FilesHelper::normalizePath($r_path);
            FilesHelper::writeFile($r_path, $file['content']);
        }

        // add version folder as root folder to archive file
        addToArchive($archive, $root, $base_upload_dir);
        FilesHelper::removeDir($root, true);
    }

    if ($includeThemler) {
        $inner_preview_dir = $base_upload_dir . '/themes/' .$userThemeName . '/' . $userThemeName . _PREVIEW_SUFFIX_;
        FilesHelper::copyRecursive($preview_template_folder, $inner_preview_dir);
    }

    foreach ($dirs as $dir)
        addToArchive($archive, $base_upload_dir . '/' . $dir, $base_upload_dir);
    clearDirs($base_upload_dir, $dirs);

    return;
}

function clearDirs($base_path, $dirs, $self = true) {
    foreach ($dirs as $dir) {
        FilesHelper::removeDir($base_path . '/' . $dir, $self);
    }
}

function add_files_to_archive_root(&$archive, $base_upload_dir, $user_template_dir, $base_preview_dir, $user_theme_name) {
    save_config($base_preview_dir, $base_upload_dir, $user_theme_name);

    $paths = array('Readme.txt' => FilesHelper::normalizePath($user_template_dir . '/Readme.txt'), 
                   'Config.xml' => FilesHelper::normalizePath($base_upload_dir   . '/designer/Export/Config.xml'));

    foreach ($paths as $name => $path) {
        if (!file_exists($path))
            continue;

        $dir = dirname($path);
        addToArchive($archive, $path, $dir);
    }
}

function save_config($src_dir, $dest_dir, $user_theme_name) {
    $config  = '/designer/Export/Config.xml';
    $content = preg_replace('/name="([^"]*?)" directory="([^"]*?)"/',
                            'name="' . $user_theme_name . '" directory="'  . $user_theme_name . '"', FilesHelper::readFile($src_dir . $config));
    FilesHelper::writeFile($dest_dir . $config, $content);
}